@@ -112,3 +112,48 @@ pub enum Either<L, R> {
     /// Right variant
     Right(R),
 }
+
+/// Enum representing partitioning choice in `PartitionThree`
+pub enum Ternary {
+    /// Variant representing first partition option
+    First,
+    /// Variant representing second partition option
+    Second,
+    /// Variant representing third partition option
+    Third,
+}
+
+/// Trait providing method analogous to `Iterator::partition`
+/// but resulting in three distinct groupings
+pub trait PartitionThree: Iterator {
+    /// Consumes an iterator, creating three collections from it.
+    /// The predicate passed to partition() can return `First`, `Second`, or `Third`. partition() returns a triple,
+    /// all of the elements for which it returned `First`, all of the elements for which it returned `Second`, and all of the elements for which it returned `Third`.
+    fn partition_three<C, F>(self, f: F) -> (C, C, C)
+    where
+        Self: Sized,
+        C: Default + Extend<<Self as Iterator>::Item>,
+        F: FnMut(&<Self as Iterator>::Item) -> Ternary,
+    {
+        #[inline]
+        fn extend<'a, T, C: Extend<T>>(
+            mut f: impl FnMut(&T) -> Ternary + 'a,
+            first: &'a mut C,
+            second: &'a mut C,
+            third: &'a mut C,
+        ) -> impl FnMut((), T) + 'a {
+            move |(), x| match f(&x) {
+                Ternary::First => first.extend([x]),
+                Ternary::Second => second.extend([x]),
+                Ternary::Third => third.extend([x]),
+            }
+        }
+
+        let mut first = C::default();
+        let mut second = C::default();
+        let mut third = C::default();
+
+        self.fold((), extend(f, &mut first, &mut second, &mut third));
+        (first, second, third)
+    }
+}
